#pragma ident "$Id: Keeper.cpp 3140 2012-06-18 15:03:02Z susancummins $"

/**
 * @file Keeper.hpp
 * This class examines a GNSS Data Structure (GDS) and keeps only specific
 * values according to their TypeIDs.
 */

//============================================================================
//
//  This file is part of GPSTk, the GPS Toolkit.
//
//  The GPSTk is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation; either version 2.1 of the License, or
//  any later version.
//
//  The GPSTk is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with GPSTk; if not, write to the Free Software Foundation,
//  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
//
//  Dagoberto Salazar - gAGE ( http://www.gage.es ). 2009, 2011
//
//============================================================================


#include "Keeper.hpp"


namespace gpstk
{

      // Returns a string identifying this object.
   std::string Keeper::getClassName() const
   { return "Keeper"; }



      /* Keeps data from a satTypeValueMap object.
       *
       * @param gData     Data object holding the data.
       */
   satTypeValueMap& Keeper::Process( satTypeValueMap& gData )
      throw(ProcessingException)
   {

      try
      {

            // Call appropriate GDS method ONLY if we have a proper set
         if( keepTypeSet.size() > 0 )
         {
            gData.keepOnlyTypeID( keepTypeSet );
         }

         return gData;

      }
      catch(Exception& u)
      {
            // Throw an exception if something unexpected happens
         ProcessingException e( getClassName() + ":"
                                + u.what() );

         GPSTK_THROW(e);

      }

   }  // End of method 'Keeper::Process()'



      /* Method to add a set of TypeIDs to be kept.
       *
       * @param keepSet       TypeIDSet of data values to be added to the
       *                      ones being kept.
       */
   Keeper& Keeper::addTypeSet( const TypeIDSet& keepSet )
   {

         // Iterate over 'keepSet' and add its components to 'keepTypeSet'
      for( TypeIDSet::const_iterator pos = keepSet.begin();
           pos != keepSet.end();
           ++pos )
      {
         keepTypeSet.insert(*pos);
      }

      return (*this);

   }  // End of method 'Keeper::addTypeSet()'


}  // End of namespace gpstk
